import threading


class Threader(threading.Thread):

    def __init__(self):
        super(Threader, self).__init__()
        self._stop = threading.Event()

    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.is_set()



import os
import struct
import subprocess
import logging
import configparser
import pathlib
from .max7219 import Symbols


class Equalizer:

    def __init__(self, display, music, equalizer_enabled):
        self._display = display
        self._music = music
        self._equalizer_enabled = equalizer_enabled
        self._process = None

        self._sample = [
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE
        ]

        self._config_file_name = str(pathlib.Path(__file__).parent.resolve()) + '/cava.config'
            
        logging.info('cava config: ' + self._config_file_name)

        config = configparser.RawConfigParser()
        config.read(self._config_file_name)

        general_bars = int(config.get('general', 'bars'))
        self._output_raw_target = config.get('output', 'raw_target')
        output_bit_format = config.get('output', 'bit_format')

        bytetype = "H" if output_bit_format == "16bit" else "B"
        bytesize = 2 if output_bit_format == "16bit" else 1
        self._bytenorm = 65535 if output_bit_format == "16bit" else 255
        self._chunk = bytesize * general_bars
        self._fmt = bytetype * general_bars

    def is_visible(self):
        return self._equalizer_enabled and self._music.is_playing()

    def run(self):
        if (self._process is None):
            self._process = subprocess.Popen(["cava", "-p", self._config_file_name], stdout=subprocess.PIPE)

            if (self._output_raw_target != "/dev/stdout"):
                if (not os.path.exists(self._output_raw_target)):
                    os.mkfifo(self._output_raw_target)
                self._source = open(self._output_raw_target, "rb")
            else:
                self._source = self._process.stdout

        self._display.draw(self._get_draw_buffer())

    def stop(self):
        if (self._process is not None):
            self._source.close()
            self._process.join()
            self._process.close()
            self._process.kill()
            self._process = None
            self._source = None

    def _get_draw_buffer(self):
        if (self._source is not None):
            data = self._source.read(self._chunk)
            if (len(data) < self._chunk):
                return self._sample
            self._sample = [self._getSymbol(i / self._bytenorm) for i in struct.unpack(self._fmt, data)]

        return self._sample

    def _getSymbol(self, ratio):
        if (ratio > 0.1):
            return Symbols.TOP
        elif (ratio > 0.4):
            return Symbols.MIDDLE
        elif (ratio > 0.8):
            return Symbols.BOTTOM
        return Symbols.NONE



import time
import logging
from subprocess import call


class IrSender:
    POWER = "power"
    VOL_DOWN = "vol-"
    VOL_UP = "vol+"
    BASS_DOWN = "bass-"
    BASS_UP = "bass+"

    def __init__(self, ir_remote, on_power):
        self._ir_remote = ir_remote
        self._on_power = on_power
        self._is_power_on = False

    def stop(self):
        self.power(False)

    def power(self, value=None):
        if (value is None or value != self._is_power_on):
            self._on_power(value)
            self._send(self._ir_remote, self.POWER)
            self._is_power_on = value

    def volume(self, value, sleep=0.2):
        command = self.VOL_UP if value > 0 else self.VOL_DOWN
        for i in range(abs(value)):
            self._send(self._ir_remote, command)
            time.sleep(sleep)

    def bass(self, value, sleep=0.2):
        command = self.BASS_UP if value > 0 else self.BASS_DOWN
        for i in range(abs(value)):
            self._send(self._ir_remote, command)
            time.sleep(sleep)

    def _send(self, remote, command):
        try:
            call(["irsend", "SEND_ONCE", remote, command])
        except Exception as inst:
            logging.error(inst)



import pykka
import logging
from mopidy.core import CoreListener
from .worker import Worker


logger = logging.getLogger(__name__)


class Frontend(pykka.ThreadingActor, CoreListener):
    worker = Worker()

    def __init__(self, config, core):
        super().__init__()

        self.config = config['sevensegmentdisplay']
        self.core = core

    def on_start(self):
        logger.warning('started')
        self.worker.start(self.config, self.core)

    def on_stop(self):
        logger.warning('stopped')
        self.worker.stop()

    def on_failure(self, exception_type, exception_value, traceback):
        logger.warning('failing')
        self.worker.stop()

    def on_event(self, event, **kwargs):
        if (event == 'stream_title_changed'):
            self.worker.on_seeked()
        return CoreListener.on_event(self, event, **kwargs)

    def playback_state_changed(self, old_state, new_state):
        self.worker.on_playback_state_changed(old_state, new_state)

    # def track_playback_started(self, tl_track):
    #     logger.warning('playback_started!')
    #     self.worker.on_playing()

    # def track_playback_paused(self, tl_track, time_position):
    #     logger.warning('playback_paused!')
    #     self.worker.on_paused()

    # def track_playback_resumed(self, tl_track, time_position):
    #     logger.warning('playback_resumed!')
    #     self.worker.on_playing()

    # def track_playback_ended(self, tl_track, time_position):
    #     logger.warning('playback_ended!')
    #     self.worker.on_stopped()

    def volume_changed(self, volume):
        self.worker.on_volume_changed(volume)

    def mute_changed(self, mute):
        self.worker.on_mute(mute)

    def seeked(self, time_position):
        self.worker.on_seeked()

    # def playlists_loaded(self):
    #     logger.warning('Received playlists_loaded event')

    # def playlist_changed(self, playlist):
    #     logger.warning('Received playlist_changed event')




class Menu:

    def __init__(self, display, menu, modules):
        self._display = display
        self._menu = menu
        self._modules = modules
        self._module_index = 0
        self._module_visible = 0
        self._sub_menu = None
        self._sub_menu_group = None
        self._sub_menu_index = 0
        self._sub_menu_visible = 0
        self.reset_sub_menu()

    def run(self):
        if (self.is_sub_menu_visible()):
            self._sub_menu_visible -= 1
            if (not self.is_sub_menu_visible()):
                self._display.draw_scroll_up_animation(self._modules[self._module_index].get_draw_buffer())
                self.reset_sub_menu()
            return

        self._run_modules()

        if (self._display.is_power_on()):
            self._draw_module()

    def reset_sub_menu(self):
        self._set_sub_menu(self._menu)

    def is_sub_menu_visible(self, seconds=0):
        if (self._sub_menu_visible <= seconds):
            return False
        else:
            return True

    def click(self):
        self._set_sub_menu_visible()

        if ("get_sub_menu" in self._sub_menu[self._sub_menu_index]):
            self._set_sub_menu(self._sub_menu[self._sub_menu_index])
            self._get_buffer_and_draw(self._display.draw_scroll_down_animation)
            self._on_draw()
        elif ("click" in self._sub_menu[self._sub_menu_index]):
            self._sub_menu[self._sub_menu_index]["click"]()
            self._click_animation()
        else:
            self._close_sub_menu()

    def click_left(self, item=None):
        self._set_sub_menu_visible()

        if (self._is_sub_menu(item)):
            return

        if ("click_left" in self._sub_menu[self._sub_menu_index]):
            self._sub_menu[self._sub_menu_index]["click_left"]()
            self._get_buffer_and_draw(self._display.draw)
        else:
            self._sub_menu_index = (self._sub_menu_index - 1) % len(self._sub_menu)
            self._get_buffer_and_draw(self._display.draw_scroll_right_animation)
            self._on_draw()

    def click_right(self, item=None):
        self._set_sub_menu_visible()

        if (self._is_sub_menu(item)):
            return

        if ("click_right" in self._sub_menu[self._sub_menu_index]):
            self._sub_menu[self._sub_menu_index]["click_right"]()
            self._get_buffer_and_draw(self._display.draw)
        else:
            self._sub_menu_index = (self._sub_menu_index + 1) % len(self._sub_menu)
            self._get_buffer_and_draw(self._display.draw_scroll_left_animation)
            self._on_draw()

    def draw_sub_menu_animation(self, anim_dict):
        self._set_sub_menu_visible(anim_dict["length"])
        self._display.draw_animation(anim_dict["buffer"], anim_dict["repeat"], anim_dict["sleep"])

    def draw_sub_menu(self, item):
        self._set_sub_menu_visible()

        if (self._is_sub_menu(item)):
            return

        self._get_buffer_and_draw(self._display.draw)

    def _run_modules(self):
        for module in self._modules:
            module.run()

    def _draw_module(self):
        if (self._modules[self._module_index].is_visible(self._module_visible)):
            self._module_visible = self._module_visible + 1
            self._display.draw(self._modules[self._module_index].get_draw_buffer())
        else:
            self._next_module_index()
            self._display.draw_scroll_left_animation(self._modules[self._module_index].get_draw_buffer())

    def _next_module_index(self):
        self._module_visible = 0
        self._module_index = (self._module_index + 1) % len(self._modules)
        for i in range(self._module_index, len(self._modules)):
            if (self._modules[i].is_visible(self._module_visible)):
                self._module_index = i
                return
        for i in range(self._module_index):
            if (self._modules[i].is_visible(self._module_visible)):
                self._module_index = i
                return

    def _get_buffer_and_draw(self, draw):
        if ("get_buffer" in self._sub_menu[self._sub_menu_index]):
            draw(self._sub_menu[self._sub_menu_index]["get_buffer"]())

    def _on_draw(self):
        if ("on_draw" in self._sub_menu[self._sub_menu_index]):
            self._sub_menu[self._sub_menu_index]["on_draw"]()

    def _click_animation(self):
        if ("click_animation" in self._sub_menu[self._sub_menu_index]):
            self._get_buffer_and_draw(self._display.draw_blink_animation)

    def _close_sub_menu(self):
        if (self._sub_menu_visible > 1):
            self._sub_menu_visible = 1

    def _is_sub_menu(self, item):
        if (item is not None and self._sub_menu_group != item["group"]):
            self._set_sub_menu(item)
            self._get_buffer_and_draw(self._display.draw_scroll_down_animation)
            return True
        return False

    def _set_sub_menu(self, item):
        self._sub_menu = item["get_sub_menu"]()
        self._sub_menu_group = item["group"] if "group" in item else None
        self._sub_menu_index = 0

    def _set_sub_menu_visible(self, seconds=5):
        self._display.set_power_on()
        self._sub_menu_visible = seconds
        self._module_index = 0
        self._module_visible = 0



import time
from .threader import Threader


class Animation(Threader):

    def __init__(self, display, buffer, repeat, sleep):
        super(Animation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._repeat = repeat
        self._sleep = sleep

    def run(self):
        for j in range(self._repeat):
            for i in range(len(self._buffer)):
                if (self.stopped()):
                    break
                self._display.set_buffer(self._buffer[i])
                self._display.flush()
                time.sleep(self._sleep)
            if (self.stopped()):
                break


class BlinkAnimation(Threader):

    def __init__(self, display, buffer, repeat, sleep):
        super(BlinkAnimation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._repeat = repeat
        self._sleep = sleep

    def run(self):
        for j in range(self._repeat):
            if (self.stopped()):
                break
            if (j % 2 == 1):
                self._display.set_buffer(self._buffer)
            else:
                self._display.clear()
            self._display.flush()
            time.sleep(self._sleep)


class ScrollLeftAnimation(Threader):

    def __init__(self, display, buffer, sleep):
        super(ScrollLeftAnimation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._sleep = sleep

    def run(self):
        for i in range(len(self._buffer)):
            if (self.stopped()):
                break
            self._display.scroll_left(self._buffer[i])
            self._display.flush()
            time.sleep(self._sleep)


class ScrollRightAnimation(Threader):

    def __init__(self, display, buffer, sleep):
        super(ScrollRightAnimation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._sleep = sleep

    def run(self):
        length = len(self._buffer)
        for i in range(length):
            if (self.stopped()):
                break
            self._display.scroll_right(self._buffer[length - i - 1])
            self._display.flush()
            time.sleep(self._sleep)


class ScrollUpAnimation(Threader):
    FROM = -1
    TO = 4

    def __init__(self, display, buffer, sleep):
        super(ScrollUpAnimation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._sleep = sleep

    def run(self):
        for i in range(self.FROM, self.TO):
            if (self.stopped()):
                break
            self._display.scroll_up(self._buffer, i)
            self._display.flush()
            time.sleep(self._sleep)


class ScrollDownAnimation(Threader):
    FROM = -1
    TO = 4

    def __init__(self, display, buffer, sleep):
        super(ScrollDownAnimation, self).__init__()
        self._display = display
        self._buffer = buffer
        self._sleep = sleep

    def run(self):
        for i in range(self.FROM, self.TO):
            if (self.stopped()):
                break
            self._display.scroll_down(self._buffer, i)
            self._display.flush()
            time.sleep(self._sleep)



import threading
import logging
from datetime import datetime
from .max7219 import SevenSegmentDisplay, Symbols
from .animation import Animation, BlinkAnimation, ScrollDownAnimation, ScrollUpAnimation, ScrollLeftAnimation, ScrollRightAnimation


class Display(object):

    def __init__(self, display_enabled):
        self._brightness = None
        self._is_shutdown = not display_enabled
        self._animation_thread = None
        self._display = SevenSegmentDisplay() if display_enabled else None
        self._lock = threading.Lock()

    def set_brightness(self, brightness):
        if (self._display is None):
            return

        if (self._brightness != brightness):
            self._brightness = brightness
            self._display.set_brightness(brightness)

    def shutdown(self, is_shutdown=True):
        if (self._display is None):
            return

        if (self._is_shutdown != is_shutdown):
            if (is_shutdown):
                self._kill_animation()
            self._is_shutdown = is_shutdown
            self._display.shutdown(is_shutdown)

    def stop(self):
        if (self._display is None):
            return
        
        self.shutdown()
        self._display.stop()
        self._display = None


    def draw(self, buffer):
        if (self._display is None):
            return
        
        self._kill_animation()
        self._display.set_buffer(buffer)
        self._display.flush()

    def draw_animation(self, buffer, repeat=1, sleep=0.05):
        if (self._display is None):
            return

        self._draw_animation(Animation(self._display, buffer, repeat, sleep))

    def draw_blink_animation(self, buffer, repeat=6, sleep=0.1):
        if (self._display is None):
            return

        self._draw_animation(BlinkAnimation(self._display, buffer, repeat, sleep))

    def draw_scroll_left_animation(self, buffer, sleep=0.05):
        if (self._display is None):
            return

        new_buffer = list(buffer)
        new_buffer.insert(0, Symbols.NONE)
        self._draw_animation(ScrollLeftAnimation(self._display, new_buffer, sleep))

    def draw_scroll_right_animation(self, buffer, sleep=0.05):
        if (self._display is None):
            return

        new_buffer = list(buffer)
        new_buffer.append(Symbols.NONE)
        self._draw_animation(ScrollRightAnimation(self._display, new_buffer, sleep))

    def draw_scroll_up_animation(self, buffer, sleep=0.05):
        if (self._display is None):
            return

        self._draw_animation(ScrollUpAnimation(self._display, buffer, sleep))

    def draw_scroll_down_animation(self, buffer, sleep=0.05):
        if (self._display is None):
            return

        self._draw_animation(ScrollDownAnimation(self._display, buffer, sleep))

    def _draw_animation(self, animation):
        self._lock.acquire()
        try:
            if (self._animation_thread is not None):
                self._animation_thread.stop()
                self._animation_thread = None
            if (animation is not None):
                self._animation_thread = animation
                self._animation_thread.start()
        except Exception as inst:
            logging.error(inst)
        finally:
            self._lock.release()

    def _kill_animation(self):
        self._draw_animation(None)


class DisplayWithPowerSaving(Display):

    def __init__(self, display_enabled, display_min_brightness, display_max_brightness, display_off_time_from, display_off_time_to):
        super(DisplayWithPowerSaving, self).__init__(display_enabled)
        self._display_min_brightness = display_min_brightness
        self._display_max_brightness = display_max_brightness
        self._display_off_time_from = display_off_time_from
        self._display_off_time_to = display_off_time_to
        self._display_power_on = datetime.now()

    def is_power_on(self):
        now = datetime.now()
        if (self._is_power_on(now)):
            self._set_power_on()
            self._set_brightness(now)
            return True
        else:
            self._set_power_off()
            return False

    def set_power_on(self):
        now = datetime.now()
        if (not self._is_power_on(now)):
            self._set_power_on()
            self._display_power_on = now

    def _is_power_on(self, now):
        return (self._display_power_on.day == now.day and
                self._display_power_on.month == now.month and
                self._display_power_on.year == now.year) or (not self._is_work_time(now))

    def _is_work_time(self, now):
        return now.weekday() < 5 and now.hour > self._display_off_time_from and now.hour < self._display_off_time_to

    # hour       - 9 10 11 12 13 14 15 16 17 18 19 20 -
    # brightness 2 3  4  5  6  7  8  8  7  6  5  4  3 2
    # when display_min_brightness=2 and display_max_brightness=8
    def _set_brightness(self, now):
        hour = now.hour
        brightness = self._display_min_brightness
        if (hour >= 9 and hour <= 20):
            if (hour < 15):
                brightness = int(round(
                    self._display_min_brightness + (self._display_max_brightness - self._display_min_brightness) / 6 * (hour - 9 + 1)))
            else:
                brightness = int(round(
                    self._display_min_brightness + (self._display_max_brightness - self._display_min_brightness) / 6 * (20 - hour + 1)))
        super(DisplayWithPowerSaving, self).set_brightness(brightness)

    def _set_power_on(self):
        super(DisplayWithPowerSaving, self).shutdown(False)

    def _set_power_off(self):
        super(DisplayWithPowerSaving, self).shutdown()



from datetime import datetime, timedelta
from .max7219 import Symbols


class Timer(object):
    VISIBLE_FOR_SECONDS = 2

    def __init__(self, callback, step, remove_min=True):
        self._callback = callback
        self._step = step
        self._half_step = int(step / 2)
        self._remove_min = remove_min
        self._now = self._datetime_now()
        self._timer = None

    def run(self):
        self._now = self._datetime_now()
        if (self.is_set()):
            if (self._is_time()):
                self._callback()
                self.reset()

    def is_set(self):
        return self._timer is not None

    def _datetime_now(self):
        return datetime.now().replace(second=0, microsecond=0)

    def _is_time(self):
        return self._now >= self._timer

    def reset(self):
        self._timer = None

    def get(self):
        return self._timer

    def set(self, hour=None, minute=None):
        if (hour is not None or minute is not None):
            new_timer = self._now
            if (hour is not None):
                new_timer = new_timer.replace(hour=int(hour))
            if (minute is not None):
                new_timer = new_timer.replace(minute=int(minute))
            if (new_timer <= self._now):
                new_timer += timedelta(days=1)
            self._timer = new_timer

    def increase(self):
        if (not self.is_set()):
            time_left = (self._step - self._now.minute % self._step) if self._remove_min else 0
            min = time_left + (self._step if time_left < self._half_step else 0)
            self._timer = self._now + timedelta(minutes=min)
        else:
            new_time = self._timer + timedelta(minutes=self._step)
            diff = new_time - self._now
            if (diff.days < 1):
                self._timer = new_time

    def decrease(self):
        if (self.is_set()):
            timer_minus_step = self._timer - timedelta(minutes=self._step)
            now_plus_half_step = self._now + timedelta(minutes=self._half_step)
            if (now_plus_half_step < timer_minus_step):
                self._timer = timer_minus_step
            else:
                self._timer = None

    def is_visible(self, seconds):
        return seconds < self.VISIBLE_FOR_SECONDS and self.is_set()


class TimerOff(Timer):
    TIMER_STEP = 30

    def __init__(self, callback):
        super(TimerOff, self).__init__(callback, self.TIMER_STEP)

    def get_draw_buffer(self):
        if (self.is_set()):
            timer = self.get()
            hour = timer.hour
            minute = timer.minute
            return [
                Symbols.O,
                Symbols.F,
                Symbols.F,
                Symbols.NONE,
                Symbols.NUMBER[int(hour / 10)],
                Symbols.NUMBER[hour % 10] + Symbols.DOT,
                Symbols.NUMBER[int(minute / 10)],
                Symbols.NUMBER[minute % 10]
            ]
        else:
            return [
                Symbols.O,
                Symbols.F,
                Symbols.F,
                Symbols.NONE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE
            ]


class TimerOn(Timer):
    TIMER_STEP = 30

    def __init__(self, callback):
        super(TimerOn, self).__init__(callback, self.TIMER_STEP)

    def get_draw_buffer(self):
        if (self.is_set()):
            timer = self.get()
            hour = timer.hour
            minute = timer.minute
            return [
                Symbols.O,
                Symbols.N,
                Symbols.NONE,
                Symbols.NONE,
                Symbols.NUMBER[int(hour / 10)],
                Symbols.NUMBER[hour % 10] + Symbols.DOT,
                Symbols.NUMBER[int(minute / 10)],
                Symbols.NUMBER[minute % 10]
            ]
        else:
            return [
                Symbols.O,
                Symbols.N,
                Symbols.NONE,
                Symbols.NONE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE
            ]


class TimerAlert:
    TIMER_STEP = 10

    def __init__(self, callback):
        self._callback = callback
        self._timer_index = 0
        self._timers = []

    def get_draw_buffer(self):
        if (len(self._timers) > self._timer_index and self._timers[self._timer_index].is_set()):
            timer = self._timers[self._timer_index].get()
            hour = timer.hour
            minute = timer.minute
            return [
                Symbols.T1,
                Symbols.T2,
                Symbols.NUMBER[self._timer_index],
                Symbols.NONE,
                Symbols.NUMBER[int(hour / 10)],
                Symbols.NUMBER[hour % 10] + Symbols.DOT,
                Symbols.NUMBER[int(minute / 10)],
                Symbols.NUMBER[minute % 10]
            ]
        else:
            return [
                Symbols.T1,
                Symbols.T2,
                Symbols.NUMBER[self._timer_index],
                Symbols.NONE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE,
                Symbols.MIDDLE
            ]

    def get_draw_menu_buffer(self):
        if (self._have_timers()):
            self._timer_index = len(self._timers) - 1
        else:
            self._timer_index = 0
        return self.get_draw_buffer()

    def run(self):
        for i in range(len(self._timers) - 1, -1, -1):
            if (not self._timers[i].is_set()):
                del self._timers[i]
            else:
                self._timers[i].run()

    def add_timer(self, hour=None, minute=None):
        if (len(self._timers) < 10):
            new_timer = Timer(self._callback, self.TIMER_STEP, False)
            if (hour is not None or minute is not None):
                new_timer.set(hour, minute)
            else:
                if (self._have_timers()):
                    timer = self._timers[len(self._timers) - 1].get()
                    new_timer.set(timer.hour, timer.minute)
                for i in range(6):
                    new_timer.increase()
            self._timers.append(new_timer)

    def _have_timers(self):
        return len(self._timers) > 0

    def is_set(self):
        for timer in self._timers:
            if (timer.is_set()):
                return True
        return False

    def reset(self):
        self._timer_index = 0
        self._timers = []

    def get(self):
        return self._timers

    def increase(self):
        if (not self._have_timers()):
            self.add_timer()
        else:
            self._timers[len(self._timers) - 1].increase()

    def decrease(self):
        if (self._have_timers()):
            self._timers[len(self._timers) - 1].decrease()

    def is_visible(self, seconds):
        self._timer_index = seconds // Timer.VISIBLE_FOR_SECONDS
        return seconds < Timer.VISIBLE_FOR_SECONDS * len(self._timers)



import RPi.GPIO as GPIO
import threading
import time
import logging
from datetime import datetime
from .max7219 import Symbols
from .threader import Threader


class Gpio:
    POWER_BUTTON_PIN = 19  # StandBy-On
    MENU_BUTTON_PIN = 13   # Open/Close
    LEFT_BUTTON_PIN = 5    # Play/Pause
    RIGHT_BUTTON_PIN = 6   # Stop
    LIGHT_SENSOR_PIN = 27
    RELAY_PIN = 4

    def __init__(self, buttons_enabled, on_power, on_menu, on_left, on_right, light_sensor_enabled, on_light, relay_enabled):
        self._lock = threading.Lock()

        GPIO.setmode(GPIO.BCM)

        if (buttons_enabled):
            GPIO.setup(self.POWER_BUTTON_PIN, GPIO.IN)
            GPIO.setup(self.MENU_BUTTON_PIN, GPIO.IN)
            GPIO.setup(self.LEFT_BUTTON_PIN, GPIO.IN)
            GPIO.setup(self.RIGHT_BUTTON_PIN, GPIO.IN)

            GPIO.add_event_detect(self.POWER_BUTTON_PIN, GPIO.RISING, bouncetime=300,
                                  callback=lambda gpio: on_power() if not self._lock.locked() else None)
            GPIO.add_event_detect(self.MENU_BUTTON_PIN, GPIO.RISING, bouncetime=300,
                                  callback=lambda gpio: on_menu() if not self._lock.locked() else None)
            GPIO.add_event_detect(self.LEFT_BUTTON_PIN, GPIO.RISING, bouncetime=300,
                                  callback=lambda gpio: on_left() if not self._lock.locked() else None)
            GPIO.add_event_detect(self.RIGHT_BUTTON_PIN, GPIO.RISING, bouncetime=300,
                                  callback=lambda gpio: on_right() if not self._lock.locked() else None)

        self._light_sensor = LightSensor(self.LIGHT_SENSOR_PIN, on_light)
        if (light_sensor_enabled):
            GPIO.setup(self.LIGHT_SENSOR_PIN, GPIO.IN)
            self._light_sensor.start()

        if (relay_enabled):
            self._is_relay_on = False
            GPIO.setup(self.RELAY_PIN, GPIO.OUT, initial=GPIO.HIGH)

        self._relay_enabled = relay_enabled

    def get_draw_sleep_animation(self):
        return self._light_sensor.ANIMATION_SLEEP

    def switch_relay(self, value):
        if (self._relay_enabled and self._lock.acquire()):
            try:
                if (value != self._is_relay_on):
                    GPIO.output(self.RELAY_PIN, GPIO.LOW if value else GPIO.HIGH)
                    self._is_relay_on = value
                    time.sleep(1)
                    return True
            except Exception as inst:
                logging.error(inst)
            finally:
                threading.Timer(1, self._lock.release).start()
        return False

    def cleanup(self):
        self._light_sensor.stop()
        GPIO.cleanup()


class LightSensor(Threader):
    S = Symbols.S
    L = Symbols.L
    E = Symbols.E
    P = Symbols.P

    ANIMATION_SLEEP = {
        "length": 2,
        "repeat": 1,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, S],
            [0, 0, 0, 0, 0, 0, S, 0],
            [0, 0, 0, 0, 0, S, 0, 0],
            [0, 0, 0, 0, S, 0, 0, 0],
            [0, 0, 0, S, 0, 0, 0, 0],
            [0, 0, S, 0, 0, 0, 0, L],
            [0, 0, S, 0, 0, 0, L, 0],
            [0, 0, S, 0, 0, L, 0, 0],
            [0, 0, S, 0, L, 0, 0, 0],
            [0, 0, S, L, 0, 0, 0, E],
            [0, 0, S, L, 0, 0, E, 0],
            [0, 0, S, L, 0, E, 0, 0],
            [0, 0, S, L, E, 0, 0, E],
            [0, 0, S, L, E, 0, E, 0],
            [0, 0, S, L, E, E, 0, 0],
            [0, 0, S, L, E, E, 0, P],
            [0, 0, S, L, E, E, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, L, E, E, P, 0],
            [0, 0, S, L, E, E, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, L, E, E, P, 0],
            [0, 0, S, L, E, E, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, L, E, E, P, 0]
        ]
    }

    def __init__(self, gpio, callback):
        super(LightSensor, self).__init__()
        self._gpio = gpio
        self._callback = callback
        self._value = 0

    def run(self):
        counter = 0
        while (True):
            if (self.stopped()):
                break
            value = GPIO.input(self._gpio)
            if (self._value != value):
                if (counter < 50):
                    counter += 1
                else:
                    counter = 0
                    self._value = value
                    self._callback(datetime.now(), True if value == 1 else False)
            else:
                counter = 0
            time.sleep(0.2)



from datetime import datetime
from .max7219 import Symbols


class Time:
    VISIBLE_FOR_SECONDS = 5

    def __init__(self):
        self._blink = True

    def run(self):
        pass

    def is_visible(self, seconds):
        return seconds < self.VISIBLE_FOR_SECONDS

    def get_draw_buffer(self):
        now = datetime.now()
        hour = now.hour
        minute = now.minute
        second = now.second

        self._blink = not self._blink

        separator = Symbols.NONE
        if (self._blink):
            separator = Symbols.MIDDLE

        time = [
            Symbols.NUMBER[int(hour / 10)],
            Symbols.NUMBER[hour % 10],
            separator,
            Symbols.NUMBER[int(minute / 10)],
            Symbols.NUMBER[minute % 10],
            separator,
            Symbols.NUMBER[int(second / 10)],
            Symbols.NUMBER[second % 10]
        ]

        return time


class Date:
    VISIBLE_FOR_SECONDS = 2

    def __init__(self, modules):
        self._modules = modules

    def run(self):
        pass

    def is_visible(self, seconds):
        return seconds < self.VISIBLE_FOR_SECONDS and not self._is_any_module_enabled()

    def get_draw_buffer(self):
        now = datetime.now()
        day = now.day
        month = now.month
        year = now.year

        date = [
            Symbols.NUMBER[int(year / 1000)],
            Symbols.NUMBER[int(year / 100) % 10],
            Symbols.NUMBER[int(year / 10) % 10],
            Symbols.NUMBER[year % 10] + Symbols.DOT,
            Symbols.NUMBER[int(month / 10)],
            Symbols.NUMBER[month % 10] + Symbols.DOT,
            Symbols.NUMBER[int(day / 10)],
            Symbols.NUMBER[day % 10]
        ]

        return date

    def _is_any_module_enabled(self):
        for module in self._modules:
            if (module.is_set()):
                return True
        return False



import logging
from time import sleep
from .equalizer import Equalizer
from .threader import Threader
from .music import Music
from .display import DisplayWithPowerSaving
from .ir import IrSender
from .gpio import Gpio
from .timer import TimerOn, TimerOff, TimerAlert
from .clock import Time, Date
from .menu import Menu
from .max7219 import Symbols
from .alert import Alert


class Worker(Threader):
    core = None
    music = None
    display = None
    gpio = None
    ir_sender = None
    timer_on = None
    timer_off = None
    alert = None
    timer_alert = None
    time = None
    date = None
    menu = None
    equalizer = None

    response_code = None

    def start(self, config, core):
        self.config = config
        self.core = core
        super(Worker, self).start()

    def run(self):
        try:
            self._init_menu()
            self.music = Music(self.core, self.config['default_tracks'], self.config['default_preset'])
            self.display = DisplayWithPowerSaving(
                self.config['display_enabled'],
                self.config['display_min_brightness'],
                self.config['display_max_brightness'],
                self.config['display_off_time_from'],
                self.config['display_off_time_to'])
            self.gpio = Gpio(
                self.config['buttons_enabled'],
                self.play_stop_music,
                self._on_menu_click,
                self._on_menu_click_left,
                self._on_menu_click_right,
                self.config['light_sensor_enabled'],
                self._on_light_sensor,
                self.config['relay_enabled'])
            self.ir_sender = IrSender(self.config['ir_remote'], self.gpio.switch_relay)
            self.timer_on = TimerOn(self.play_music)
            self.timer_off = TimerOff(self.stop_music)
            self.alert = Alert(self.music,
                               self.ir_sender,
                               self.config['alert_files'])
            self.timer_alert = TimerAlert(self.run_alert)
            self.time = Time()
            self.date = Date([self.timer_on, self.timer_off, self.timer_alert])
            self.menu = Menu(
                self.display,
                self.MENU,
                [self.time, self.date, self.timer_on, self.timer_off, self.timer_alert])
            self.equalizer = Equalizer(self.display, self.music, self.config['equalizer_enabled'])

            while True:
                if (self.stopped()):
                    break

                if (self.equalizer.is_visible() and not self.menu.is_sub_menu_visible()):
                    self.equalizer.run()
                else:
                    self.equalizer.stop()
                    self.menu.run()
                    sleep(1)

        except Exception as inst:
            logging.error(inst)
        finally:
            self.equalizer.stop()
            self.ir_sender.stop()
            self.display.stop()
            self.gpio.cleanup()

    def _init_menu(self):
        self.MENU = {
            "get_sub_menu": lambda: [
                {
                    "get_sub_menu": lambda: [
                        self.MENU_TIMER,
                        self.MENU_PLAY_1,
                        self.MENU_RUN_SH,
                        self.MENU_VOLUME,
                        self.MENU_STYLE,
                        self.MENU_DEMO,
                        self.MENU_SYSTEM
                    ]
                }
            ]
        }
        self.MENU_TIMER = {
            "get_buffer": lambda: [0, Symbols.T1, Symbols.T2, Symbols.I, Symbols.M1, Symbols.M2, Symbols.E, Symbols.R],
            "get_sub_menu": lambda: [
                self.MENU_ALERT,
                self.MENU_TIMER_OFF,
                self.MENU_TIMER_ON
            ]
        }
        self.MENU_ALERT = {
            "get_buffer": lambda: [0, Symbols.A, Symbols.L, Symbols.E, Symbols.R, Symbols.T1, Symbols.T2, 0],
            "get_sub_menu": lambda: [
                self.MENU_ALERT_ADD,
                self.MENU_ALERT_CLEAR,
                self.MENU_ALERT_RUN
            ]
        }
        self.MENU_ALERT_ADD = {
            "get_buffer": lambda: [0, 0, 0, Symbols.A, Symbols.D, Symbols.D, 0, 0],
            "get_sub_menu": lambda: [
                {
                    "get_buffer": self.timer_alert.get_draw_menu_buffer,
                    "click": lambda: (self.timer_alert.add_timer(),
                                      self.display.draw(self.timer_alert.get_draw_menu_buffer())),
                    "click_left": self.timer_alert.decrease,
                    "click_right": self.timer_alert.increase
                }
            ]
        }
        self.MENU_ALERT_CLEAR = {
            "get_buffer": lambda: [0, 0, Symbols.C, Symbols.L, Symbols.E, Symbols.A, Symbols.R, 0],
            "click": lambda: self.timer_alert.reset(),
            "click_animation": True
        }
        self.MENU_ALERT_RUN = {
            "get_buffer": lambda: [0, 0, 0, Symbols.R, Symbols.U, Symbols.N, 0, 0],
            "click": lambda: self.run_alert(),
            "click_animation": True
        }
        self.MENU_TIMER_OFF = {
            "group": "timer_off",
            "get_buffer": lambda: [0, 0, 0, Symbols.O, Symbols.F, Symbols.F, 0, 0],
            "get_sub_menu": lambda: [
                {
                    "get_buffer": self.timer_off.get_draw_buffer,
                    "click_left": self.timer_off.decrease,
                    "click_right": self.timer_off.increase
                }
            ]
        }
        self.MENU_TIMER_ON = {
            "group": "timer_on",
            "get_buffer": lambda: [0, 0, 0, Symbols.O, Symbols.N, 0, 0, 0],
            "get_sub_menu": lambda: [
                {
                    "get_buffer": self.timer_on.get_draw_buffer,
                    "click_left": self.timer_on.decrease,
                    "click_right": self.timer_on.increase
                }
            ]
        }
        self.MENU_PLAY_1 = {
            "get_buffer": lambda: [0, Symbols.P, Symbols.L, Symbols.A, Symbols.Y, 0, Symbols.NUMBER[1], 0],
            "click": lambda: self.play_music(self.music.get_default_tracks())
        }
        self.MENU_VOLUME = {
            "group": "volume",
            "get_buffer": lambda: [0, Symbols.U, Symbols.O, Symbols.L, Symbols.U, Symbols.M1, Symbols.M2, Symbols.E],
            "get_sub_menu": lambda: [
                {
                    "get_buffer": self.music.get_draw_volume,
                    "click_left": self.music.decrease_volume,
                    "click_right": self.music.increase_volume
                }
            ]
        }
        self.MENU_STYLE = {
            "group": "style",
            "get_buffer": lambda: [0, Symbols.S, Symbols.T1, Symbols.T2, Symbols.Y, Symbols.L, Symbols.E, 0],
            "get_sub_menu": lambda: list(map(lambda x: {
                "get_buffer": lambda: x["buffer"],
                "on_draw": lambda: self.music.set_preset(x["name"])
            }, self.music.get_presets()))
        }
        self.MENU_RUN_SH = {
            "get_buffer": lambda: [0, Symbols.R, Symbols.U, Symbols.N, 0, Symbols.S, Symbols.H, 0],
            "click": lambda: self.music.run_sh(),
            "click_animation": True
        }
        self.MENU_DEMO = {
            "get_buffer": lambda: [0, 0, Symbols.D, Symbols.E, Symbols.M1, Symbols.M2, Symbols.O, 0],
            "click": lambda: (self.menu.draw_sub_menu_animation(self.music.get_draw_equalizer_animation()),
                              self.menu.reset_sub_menu())
        }
        self.MENU_SYSTEM = {
            "get_buffer": lambda: [Symbols.S, Symbols.Y, Symbols.S, Symbols.T1, Symbols.T2, Symbols.E, Symbols.M1, Symbols.M2],
            "get_sub_menu": lambda: [
                self.MENU_REBOOT,
                self.MENU_HALT
            ]
        }
        self.MENU_REBOOT = {
            "get_buffer": lambda: [0, Symbols.R, Symbols.E, Symbols.B, Symbols.O, Symbols.O, Symbols.T1, Symbols.T2],
            "click": lambda: self.music.reboot(),
            "click_animation": True
        }
        self.MENU_HALT = {
            "get_buffer": lambda: [0, 0, Symbols.H, Symbols.A, Symbols.L, Symbols.T1, Symbols.T2, 0],
            "click": lambda: self.music.halt(),
            "click_animation": True
        }

    def _on_menu_click(self):
        self.menu.click()

    def _on_menu_click_left(self):
        if (self.menu.is_sub_menu_visible()):
            self.menu.click_left()
        else:
            self._decrease_timer()

    def _on_menu_click_right(self):
        if (self.menu.is_sub_menu_visible()):
            self.menu.click_right()
        else:
            self._increase_timer()

    def _on_light_sensor(self, now, is_dark):
        if (self.music.is_playing() and
            (now.hour >= self.config['light_sensor_time_from'] or now.hour < self.config['light_sensor_time_to'])):
            if (is_dark):
                if (self.music.get_volume() > self.config['light_sensor_volume']):
                    self.music.set_volume(self.config['light_sensor_volume'])
                self.music.set_preset(self.config['light_sensor_preset'])
                self.timer_off.reset()
                self.timer_off.increase()
                self.timer_off.increase()
                self.timer_off.increase()
                self.timer_off.increase()
                self.menu.draw_sub_menu_animation(self.gpio.get_draw_sleep_animation())
            else:
                self.timer_off.reset()

    def _on_change_preset(self, value):
        self.music.set_preset(value)
        if (value < 0):
            self.menu.click_left(self.MENU_STYLE)
        else:
            self.menu.click_right(self.MENU_STYLE)

    def _increase_timer(self):
        if (self.music.is_playing()):
            self.timer_off.increase()
            self.menu.draw_sub_menu(self.MENU_TIMER_OFF)
        else:
            self.timer_on.increase()
            self.menu.draw_sub_menu(self.MENU_TIMER_ON)

    def _decrease_timer(self):
        if (self.music.is_playing()):
            self.timer_off.decrease()
            self.menu.draw_sub_menu(self.MENU_TIMER_OFF)
        else:
            self.timer_on.decrease()
            self.menu.draw_sub_menu(self.MENU_TIMER_ON)

    def run_alert(self):
        self.menu.draw_sub_menu_animation(self.alert.get_draw_alert_animation())
        self.alert.run()

    def get_presets(self):
        return self.music.get_presets()

    def set_preset(self, value):
        self.music.set_preset(value)

    def get_volume(self):
        return self.music.get_volume()

    def set_volume(self, volume):
        self.music.set_volume(volume)

    def get_state(self):
        return self.music.get_state()

    def play_stop_music(self):
        if (self.music.is_playing()):
            self.stop_music()
        else:
            self.play_music()

    def play_music(self, tracks=None):
        self.music.set_volume(self.config['default_volume'])
        self.music.set_preset(self.config['default_preset'])
        self.music.play(tracks)
        self.on_started()

    def pause_music(self):
        self.music.pause()
        self.on_paused()

    def stop_music(self):
        self.music.stop()
        self.on_stopped()

    def on_started(self):
        self.menu.draw_sub_menu_animation(self.music.get_draw_start_animation())

    def on_stopped(self):
        self.menu.draw_sub_menu_animation(self.music.get_draw_stop_animation())
        self.timer_off.reset()
        self.ir_sender.power(False)

    def on_playing(self):
        self.menu.draw_sub_menu_animation(self.music.get_draw_play_animation())
        self.timer_on.reset()
        if (self.music.is_playing()):
            self.ir_sender.power(True)

    def on_paused(self):
        self.menu.draw_sub_menu_animation(self.music.get_draw_pause_animation())

    def on_seeked(self):
        self.menu.draw_sub_menu_animation(self.music.get_draw_seek_animation())

    def on_mute(self, mute):
        if mute:
            self.on_volume_changed(0)
        else:
            self.on_volume_changed()

    def on_volume_changed(self, volume=None):
        if (self.menu is not None and self.music is not None and self.music.is_volume_changed(volume)):
            self.menu.draw_sub_menu(self.MENU_VOLUME)

    def on_playback_state_changed(self, old_state, new_state):
        if (old_state != new_state):
            if (self.music.is_playing(new_state)):
                self.on_playing()
            elif (self.music.is_paused(new_state)):
                self.on_paused()




class Constants(object):
    MAX7219_REG_NOOP = 0x0
    MAX7219_REG_DIGIT0 = 0x1
    MAX7219_REG_DIGIT1 = 0x2
    MAX7219_REG_DIGIT2 = 0x3
    MAX7219_REG_DIGIT3 = 0x4
    MAX7219_REG_DIGIT4 = 0x5
    MAX7219_REG_DIGIT5 = 0x6
    MAX7219_REG_DIGIT6 = 0x7
    MAX7219_REG_DIGIT7 = 0x8
    MAX7219_REG_DECODEMODE = 0x9
    MAX7219_REG_INTENSITY = 0xA
    MAX7219_REG_SCANLIMIT = 0xB
    MAX7219_REG_SHUTDOWN = 0xC
    MAX7219_REG_DISPLAYTEST = 0xF


class Symbols(object):
    NUMBER = [0x7e, 0x30, 0x6d, 0x79, 0x33, 0x5b, 0x5f, 0x70, 0x7f, 0x7b]
    BOTTOM = 0x08
    TOP = 0x40
    LEFT_TOP = 0x02
    LEFT_BOTTOM = 0x04
    RIGHT_TOP = 0x20
    RIGHT_BOTTOM = 0x10
    MIDDLE = 0x1
    NONE = 0
    DOT = 0x80
    A = int('01110111', 2)
    B = int('00011111', 2)
    C = int('01001110', 2)
    D = int('00111101', 2)
    E = int('01001111', 2)
    F = int('01000111', 2)
    # G
    H = int('00110111', 2)
    I = int('00110000', 2)
    J = int('00111100', 2)
    # K
    L = int('00001110', 2)
    M1 = int('00010101', 2)
    M2 = int('00010001', 2)
    N = int('00010101', 2)
    O = int('01111110', 2)
    P = int('01100111', 2)
    # Q
    R = int('00000101', 2)
    S = int('01011011', 2)
    T1 = int('01000000', 2)
    T2 = int('01000110', 2)
    U = int('00111110', 2)
    # V
    W1 = int('00011100', 2)
    W2 = int('00011000', 2)
    # X
    Y = int('00110011', 2)
    # Z


class SevenSegmentDisplay(object):
    NUM_DIGITS = 8

    def __init__(self, cascaded=1, spi_bus=0, spi_device=0, brightness=0):
        import spidev
        assert cascaded > 0, "Must have at least one device!"

        self._cascaded = cascaded
        self._buffer = [0] * self.NUM_DIGITS * self._cascaded
        self._spi = spidev.SpiDev()
        self._spi.open(spi_bus, spi_device)
        self._spi.cshigh = False
        self._spi.max_speed_hz = 8000000

        self.command(Constants.MAX7219_REG_SCANLIMIT, 7)    # show all 8 digits
        self.command(Constants.MAX7219_REG_DECODEMODE, 0)   # use matrix (not digits)
        self.command(Constants.MAX7219_REG_DISPLAYTEST, 0)  # no display test
        self.shutdown(False)                                # not shutdown mode
        self.set_brightness(brightness)                     # intensity: range: 0..15
        self.clear()

    def shutdown(self, value):
        self.command(Constants.MAX7219_REG_SHUTDOWN, 0 if value else 1)

    def stop(self):
        self._spi.close()

    def command(self, register, data):
        assert Constants.MAX7219_REG_DECODEMODE <= register <= Constants.MAX7219_REG_DISPLAYTEST

        self._write([register, data] * self._cascaded)

    def _write(self, data):
        self._spi.xfer2(list(data))

    def _values(self, position, buf):
        for deviceId in range(self._cascaded):
            yield position + Constants.MAX7219_REG_DIGIT0
            yield buf[(deviceId * self.NUM_DIGITS) + position]

    def clear(self, deviceId=None):
        assert not deviceId or 0 <= deviceId < self._cascaded, "Invalid deviceId: {0}".format(deviceId)

        if deviceId is None:
            start = 0
            end = self._cascaded
        else:
            start = deviceId
            end = deviceId + 1

        for deviceId in range(start, end):
            for position in range(self.NUM_DIGITS):
                self.set_byte(deviceId, position + Constants.MAX7219_REG_DIGIT0, 0)

        self.flush()

    def flush(self):
        buf = list(self._buffer)
        for posn in range(self.NUM_DIGITS):
            self._write(self._values(posn, buf))

    def set_brightness(self, intensity):
        assert 0 <= intensity < 16, "Invalid brightness: {0}".format(intensity)

        self.command(Constants.MAX7219_REG_INTENSITY, intensity)

    def set_byte(self, deviceId, position, value):
        assert 0 <= deviceId < self._cascaded, "Invalid deviceId: {0}".format(deviceId)
        assert Constants.MAX7219_REG_DIGIT0 <= position <= Constants.MAX7219_REG_DIGIT7, "Invalid digit/column: {0}".format(position)
        assert 0 <= value < 256, 'Value {0} outside range 0..255'.format(value)

        offset = (deviceId * self.NUM_DIGITS) + position - Constants.MAX7219_REG_DIGIT0
        self._buffer[offset] = value

    def set_buffer(self, buffer):
        assert len(buffer) == len(self._buffer), "Buffer is wrong size"

        length = len(buffer)
        for deviceId in range(0, self._cascaded):
            for position in range(self.NUM_DIGITS):
                pos = (deviceId * self.NUM_DIGITS) + position
                self.set_byte(deviceId, length - pos, buffer[pos])

    def scroll_left(self, value=0):
        assert 0 <= value < 256, 'Value {0} outside range 0..255'.format(value)

        del self._buffer[-1]
        self._buffer.insert(0, value)

    def scroll_right(self, value=0):
        assert 0 <= value < 256, 'Value {0} outside range 0..255'.format(value)

        del self._buffer[0]
        self._buffer.append(value)

    def scroll_down(self, buffer=None, index=0):
        if (buffer is not None):
            assert len(buffer) == len(self._buffer), "Buffer is wrong size"

        length = len(self._buffer)
        for i in range(length):
            byte = 0

            if (index < 3):
                byte |= self._switch_bit(self._buffer[i], 0, 3)
                byte |= self._switch_bit(self._buffer[i], 6, 0)
                byte |= self._switch_bit(self._buffer[i], 5, 4)
                byte |= self._switch_bit(self._buffer[i], 1, 2)

            if (buffer is not None):
                pos = length - i - 1
                if (index == 1):
                    byte |= self._switch_bit(buffer[pos], 3, 6)
                elif (index == 2):
                    byte |= self._switch_bit(buffer[pos], 0, 6)
                    byte |= self._switch_bit(buffer[pos], 3, 0)
                    byte |= self._switch_bit(buffer[pos], 4, 5)
                    byte |= self._switch_bit(buffer[pos], 2, 1)
                elif (index >= 3):
                    byte = buffer[pos]

            self._buffer[i] = byte

    def scroll_up(self, buffer=None, index=0):
        if (buffer is not None):
            assert len(buffer) == len(self._buffer), "Buffer is wrong size"

        length = len(self._buffer)
        for i in range(length):
            byte = 0

            if (index < 3):
                byte |= self._switch_bit(self._buffer[i], 3, 0)
                byte |= self._switch_bit(self._buffer[i], 0, 6)
                byte |= self._switch_bit(self._buffer[i], 4, 5)
                byte |= self._switch_bit(self._buffer[i], 2, 1)

            if (buffer is not None):
                pos = length - i - 1
                if (index == 1):
                    byte |= self._switch_bit(buffer[pos], 6, 3)
                elif (index == 2):
                    byte |= self._switch_bit(buffer[pos], 6, 0)
                    byte |= self._switch_bit(buffer[pos], 0, 3)
                    byte |= self._switch_bit(buffer[pos], 5, 4)
                    byte |= self._switch_bit(buffer[pos], 1, 2)
                elif (index >= 3):
                    byte = buffer[pos]

            self._buffer[i] = byte

    def _switch_bit(self, number, index_src, index_des):
        bit_value = ((number >> index_src) & 1) << index_des
        return bit_value



import os
import tornado.template
import tornado.web


template_directory = os.path.join(os.path.dirname(__file__), 'templates')
template_loader = tornado.template.Loader(template_directory)


class BaseRequestHandler(tornado.web.RequestHandler):
    def initialize(self, config, core, worker):
        self.config = config
        self.core = core
        self.worker = worker

    def send_message(self, code):
        self.worker.response_code = code
        self.redirect('/sevensegmentdisplay/')


class MainRequestHandler(BaseRequestHandler):
    def get(self):
        self.write(template_loader.load('index.html').generate(
            config=self.config,
            core=self.core,
            worker=self.worker
        ))


class ApiRequestHandler(BaseRequestHandler):
    def post(self):
        state = str(self.get_argument('state', ''))
        if (state == 'play'):
            self.worker.play_music()
        elif (state == 'pause'):
            self.worker.pause_music()
        elif (state == 'stop'):
            self.worker.stop_music()
        elif (state == 'play_stop'):
            self.worker.play_stop_music()

        volume = int(self.get_argument('volume', 0))
        if (volume >= 1 and volume <= 100):
            self.worker.set_volume(volume)

        off = str(self.get_argument('off', ''))
        if (off == 'set'):
            hour = self.get_argument('hour', None)
            minute = self.get_argument('minute', None)
            self.worker.timer_off.set(hour, minute)
        elif (off == '-'):
            self.worker.timer_off.decrease()
        elif (off == '+'):
            self.worker.timer_off.increase()

        on = str(self.get_argument('on', ''))
        if (on == 'set'):
            hour = self.get_argument('hour', None)
            minute = self.get_argument('minute', None)
            self.worker.timer_on.set(hour, minute)
        elif (on == '-'):
            self.worker.timer_on.decrease()
        elif (on == '+'):
            self.worker.timer_on.increase()

        alert = str(self.get_argument('alert', ''))
        if (alert == 'add'):
            hour = self.get_argument('hour', None)
            minute = self.get_argument('minute', None)
            self.worker.timer_alert.add_timer(hour, minute)
        elif (alert == 'clear'):
            self.worker.timer_alert.reset()
        elif (alert == '-'):
            self.worker.timer_alert.decrease()
        elif (alert == '+'):
            self.worker.timer_alert.increase()
        elif (alert == 'run'):
            self.worker.run_alert()

        preset = str(self.get_argument('preset', ''))
        if (preset != ''):
            self.worker.set_preset(preset)

        self.write(str(self.worker.get_volume()))
        self.write(self.worker.get_state())


def factory_decorator(worker):
    def app_factory(config, core):
        # since all the RequestHandler-classes get the same arguments ...
        def bind(url, klass):
            return (url, klass, {'config': config['sevensegmentdisplay'], 'core': core, 'worker': worker})

        return [
            (r'/static/(.*)', tornado.web.StaticFileHandler, {'path': os.path.join(os.path.dirname(__file__), 'static')}),
            bind('/', MainRequestHandler),
            bind('/api/', ApiRequestHandler)
        ]

    return app_factory



import os
import random
import time
import logging
import json
from subprocess import call
from .max7219 import Symbols


class Alert:
    A = Symbols.A
    L = Symbols.L
    E = Symbols.E
    R = Symbols.R
    T1 = Symbols.T1
    T2 = Symbols.T2

    ANIMATION_ALERT = {
        "length": 1,
        "repeat": 1,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, A],
            [0, 0, 0, 0, 0, 0, A, 0],
            [0, 0, 0, 0, 0, A, 0, 0],
            [0, 0, 0, 0, A, 0, 0, 0],
            [0, 0, 0, A, 0, 0, 0, 0],
            [0, 0, A, 0, 0, 0, 0, 0],
            [0, A, 0, 0, 0, 0, 0, L],
            [0, A, 0, 0, 0, 0, L, 0],
            [0, A, 0, 0, 0, L, 0, 0],
            [0, A, 0, 0, L, 0, 0, 0],
            [0, A, 0, L, 0, 0, 0, 0],
            [0, A, L, 0, 0, 0, 0, E],
            [0, A, L, 0, 0, 0, E, 0],
            [0, A, L, 0, 0, E, 0, 0],
            [0, A, L, 0, E, 0, 0, 0],
            [0, A, L, E, 0, 0, 0, R],
            [0, A, L, E, 0, 0, R, 0],
            [0, A, L, E, 0, R, 0, 0],
            [0, A, L, E, R, 0, 0, T1],
            [0, A, L, E, R, 0, T1, T2],
            [0, A, L, E, R, T1, T2, 0],
            [0, A, L, E, R, T1, T2, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, A, L, E, R, T1, T2, 0],
            [0, A, L, E, R, T1, T2, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, A, L, E, R, T1, T2, 0],
            [0, A, L, E, R, T1, T2, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, A, L, E, R, T1, T2, 0]
        ]
    }

    def __init__(self, music, ir_sender, files):
        self._music = music
        self._ir_sender = ir_sender
        self._files = json.loads(files)

    def run(self):
        try:
            file = random.choice(filter(lambda x: x["enabled"], self._files))

            if (not self._music.is_playing()):
                self._ir_sender.power(True)
                time.sleep(10 if "ir_send" in file else 2)

            if ("ir_send" in file):
                if ("bass" in file["ir_send"]):
                    self._ir_sender.bass(file["ir_send"]["bass"])
                if ("volume" in file["ir_send"]):
                    self._ir_sender.volume(file["ir_send"]["volume"])

            self._play_file(file["name"], file["volume"], file["repeat"])

            if ("ir_send" in file):
                if ("bass" in file["ir_send"]):
                    self._ir_sender.bass(- file["ir_send"]["bass"])
                if ("volume" in file["ir_send"]):
                    self._ir_sender.volume(- file["ir_send"]["volume"])

            if (not self._music.is_playing()):
                time.sleep(5 if "ir_send" in file else 1)
                self._ir_sender.power(False)
        except Exception as inst:
            logging.error(inst)

    def get_draw_alert_animation(self):
        return self.ANIMATION_ALERT

    def _play_file(self, file, volume=32768, repeat=1, sleep=0.5):
        for i in range(repeat):
            call(["mpg123", "-f", str(volume), "-q", os.path.join(os.path.dirname(__file__), file)])
            time.sleep(sleep)



import re

from setuptools import find_packages, setup


def get_version(filename):
    content = open(filename).read()
    metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", content))
    return metadata['version']


setup(
    name='Mopidy-SevenSegmentDisplay',
    version=get_version('mopidy_sevensegmentdisplay/__init__.py'),
    url='https://github.com/JumalIO/mopidy-sevensegmentdisplay',
    license='Apache License, Version 2.0',
    author='Julius',
    author_email='spamjulius@mail.com',
    maintainer='Julius',
    maintainer_email='spamjulius@mail.com',
    description='A Mopidy extension for using it with seven segment display.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=['tests', 'tests.*']),
    zip_safe=False,
    include_package_data=True,
    python_requires='>= 3.7',
    install_requires=[
        'setuptools',
        'Mopidy >= 3.0',
        'Pykka >= 1.1',
        'monotonic >= 1.4',
    ],
    test_suite='nose.collector',
    tests_require=[
        'nose',
        'mock >= 1.0',
    ],
    entry_points={
        'mopidy.ext': [
            'sevensegmentdisplay = mopidy_sevensegmentdisplay:Extension',
        ],
    },
    classifiers=[
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Multimedia :: Sound/Audio :: Players',
    ],
)



import os
from mopidy import config, ext
from .http import factory_decorator
from .actor import Frontend

__version__ = '0.6.1'


class Extension(ext.Extension):
    dist_name = 'Mopidy-SevenSegmentDisplay'
    ext_name = 'sevensegmentdisplay'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['display_enabled'] = config.Boolean()
        schema['buttons_enabled'] = config.Boolean()
        schema['light_sensor_enabled'] = config.Boolean()
        schema['relay_enabled'] = config.Boolean()
        schema['ir_remote'] = config.String()
        schema['default_tracks'] = config.String()
        schema['default_volume'] = config.Integer()
        schema['default_preset'] = config.String()
        schema['light_sensor_volume'] = config.Integer()
        schema['light_sensor_preset'] = config.String()
        schema['light_sensor_time_from'] = config.Integer()
        schema['light_sensor_time_to'] = config.Integer()
        schema['alert_files'] = config.String(optional=True)
        schema['display_min_brightness'] = config.Integer()
        schema['display_max_brightness'] = config.Integer()
        schema['display_off_time_from'] = config.Integer()
        schema['display_off_time_to'] = config.Integer()
        schema['equalizer_enabled'] = config.Boolean()
        return schema

    def setup(self, registry):
        registry.add('frontend', Frontend)
        registry.add('http:app', {
            'name': self.ext_name,
            'factory': factory_decorator(Frontend.worker),
        })



from mopidy import core
from random import randint
import logging
import os
from subprocess import call
from .max7219 import Symbols


class Music:
    P = Symbols.P
    L = Symbols.L
    A = Symbols.A
    Y = Symbols.Y
    S = Symbols.S
    T1 = Symbols.T1
    T2 = Symbols.T2
    O = Symbols.O
    U = Symbols.U
    E = Symbols.E

    ANIMATION_PLAY = {
        "length": 2,
        "repeat": 1,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, P],
            [0, 0, 0, 0, 0, 0, P, 0],
            [0, 0, 0, 0, 0, P, 0, 0],
            [0, 0, 0, 0, P, 0, 0, 0],
            [0, 0, 0, P, 0, 0, 0, 0],
            [0, 0, P, 0, 0, 0, 0, L],
            [0, 0, P, 0, 0, 0, L, 0],
            [0, 0, P, 0, 0, L, 0, 0],
            [0, 0, P, 0, L, 0, 0, 0],
            [0, 0, P, L, 0, 0, 0, A],
            [0, 0, P, L, 0, 0, A, 0],
            [0, 0, P, L, 0, A, 0, 0],
            [0, 0, P, L, A, 0, 0, Y],
            [0, 0, P, L, A, 0, Y, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, P, L, A, Y, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, L, A, Y, 0, 0]
        ]
    }

    ANIMATION_STOP = {
        "length": 2,
        "repeat": 1,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, S],
            [0, 0, 0, 0, 0, 0, S, 0],
            [0, 0, 0, 0, 0, S, 0, 0],
            [0, 0, 0, 0, S, 0, 0, 0],
            [0, 0, 0, S, 0, 0, 0, 0],
            [0, 0, S, 0, 0, 0, 0, T1],
            [0, 0, S, 0, 0, 0, T1, T2],
            [0, 0, S, 0, 0, T1, T2, 0],
            [0, 0, S, 0, T1, T2, 0, 0],
            [0, 0, S, T1, T2, 0, 0, O],
            [0, 0, S, T1, T2, 0, O, 0],
            [0, 0, S, T1, T2, O, 0, P],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, S, T1, T2, O, P, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, S, T1, T2, O, P, 0]
        ]
    }

    ANIMATION_PAUSE = {
        "length": 2,
        "repeat": 1,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, P],
            [0, 0, 0, 0, 0, 0, P, 0],
            [0, 0, 0, 0, 0, P, 0, 0],
            [0, 0, 0, 0, P, 0, 0, 0],
            [0, 0, 0, P, 0, 0, 0, 0],
            [0, 0, P, 0, 0, 0, 0, A],
            [0, 0, P, 0, 0, 0, A, 0],
            [0, 0, P, 0, 0, A, 0, 0],
            [0, 0, P, 0, A, 0, 0, 0],
            [0, 0, P, A, 0, 0, 0, U],
            [0, 0, P, A, 0, 0, U, 0],
            [0, 0, P, A, 0, U, 0, 0],
            [0, 0, P, A, U, 0, 0, S],
            [0, 0, P, A, U, 0, S, 0],
            [0, 0, P, A, U, S, 0, E],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, P, A, U, S, E, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, P, A, U, S, E, 0]
        ]
    }

    BOTTOM = Symbols.BOTTOM
    TOP = Symbols.TOP
    LEFT_TOP = Symbols.LEFT_TOP
    LEFT_BOTTOM = Symbols.LEFT_BOTTOM
    RIGHT_TOP = Symbols.RIGHT_TOP
    RIGHT_BOTTOM = Symbols.RIGHT_BOTTOM

    ANIMATION_SPINNER = {
        "length": 1,
        "repeat": 3,
        "sleep": 0.05,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, BOTTOM],
            [0, 0, 0, 0, 0, 0, BOTTOM, 0],
            [0, 0, 0, 0, 0, BOTTOM, 0, 0],
            [0, 0, 0, 0, BOTTOM, 0, 0, 0],
            [0, 0, 0, BOTTOM, 0, 0, 0, 0],
            [0, 0, BOTTOM, 0, 0, 0, 0, 0],
            [0, BOTTOM, 0, 0, 0, 0, 0, 0],
            [BOTTOM, 0, 0, 0, 0, 0, 0, 0],
            [LEFT_BOTTOM, 0, 0, 0, 0, 0, 0, 0],
            [LEFT_TOP, 0, 0, 0, 0, 0, 0, 0],
            [TOP, 0, 0, 0, 0, 0, 0, 0],
            [0, TOP, 0, 0, 0, 0, 0, 0],
            [0, 0, TOP, 0, 0, 0, 0, 0],
            [0, 0, 0, TOP, 0, 0, 0, 0],
            [0, 0, 0, 0, TOP, 0, 0, 0],
            [0, 0, 0, 0, 0, TOP, 0, 0],
            [0, 0, 0, 0, 0, 0, TOP, 0],
            [0, 0, 0, 0, 0, 0, 0, TOP],
            [0, 0, 0, 0, 0, 0, 0, RIGHT_TOP],
            [0, 0, 0, 0, 0, 0, 0, RIGHT_BOTTOM]
        ]
    }

    MIDDLE = Symbols.MIDDLE

    ANIMATION_START = {
        "length": 5,
        "repeat": 20,
        "sleep": 0.02,
        "buffer": [
            [0, 0, 0, 0, 0, 0, 0, MIDDLE],
            [0, 0, 0, 0, 0, 0, MIDDLE, 0],
            [0, 0, 0, 0, 0, MIDDLE, 0, 0],
            [0, 0, 0, 0, MIDDLE, 0, 0, 0],
            [0, 0, 0, MIDDLE, 0, 0, 0, 0],
            [0, 0, MIDDLE, 0, 0, 0, 0, 0],
            [0, MIDDLE, 0, 0, 0, 0, 0, 0],
            [MIDDLE, 0, 0, 0, 0, 0, 0, 0],
            [0, MIDDLE, 0, 0, 0, 0, 0, 0],
            [0, 0, MIDDLE, 0, 0, 0, 0, 0],
            [0, 0, 0, MIDDLE, 0, 0, 0, 0],
            [0, 0, 0, 0, MIDDLE, 0, 0, 0],
            [0, 0, 0, 0, 0, MIDDLE, 0, 0],
            [0, 0, 0, 0, 0, 0, MIDDLE, 0]
        ]
    }

    M0 = int('00000000', 2)
    M1 = int('00001000', 2)
    M2 = int('00001001', 2)
    M3 = int('01001001', 2)
    MUSIC_EQUALIZER = [M0, M1, M2, M3]

    N = Symbols.N
    B = Symbols.NUMBER[8]
    F = Symbols.F
    C = Symbols.C
    I = Symbols.I
    D = Symbols.D
    H = Symbols.H
    R = Symbols.R
    b = Symbols.B
    G = Symbols.NUMBER[6]

    PRESETS = [
        {"name": "flat", "curve": [65, 65, 65, 65, 65, 65, 65, 65, 65, 65], "buffer": [0, F, L, A, T1, T2, 0, 0]},
        {"name": "classical", "curve": [71, 71, 71, 71, 71, 71, 84, 83, 83, 87], "buffer": [C, L, A, S, S, I, C, 0]},
        {"name": "club", "curve": [71, 71, 67, 63, 63, 63, 67, 71, 71, 71], "buffer": [0, 0, C, L, U, B, 0, 0]},
        {"name": "dance", "curve": [57, 61, 69, 71, 71, 81, 83, 83, 71, 71], "buffer": [0, D, A, N, C, E, 0, 0]},
        {"name": "headphones", "curve": [65, 55, 64, 77, 75, 70, 65, 57, 52, 49], "buffer": [H, E, A, D, P, H, O, N]},
        {"name": "bass", "curve": [59, 59, 59, 63, 70, 78, 85, 88, 89, 89], "buffer": [0, 0, B, A, S, S, 0, 0]},
        {"name": "treble", "curve": [87, 87, 87, 78, 68, 55, 47, 47, 47, 45], "buffer": [T1, T2, R, E, b, L, E, 0]},
        {"name": "large_hall", "curve": [56, 56, 63, 63, 71, 79, 79, 79, 71, 71], "buffer": [0, 0, H, A, L, L, 0, 0]},
        {"name": "live", "curve": [79, 71, 66, 64, 63, 63, 66, 68, 68, 69], "buffer": [0, 0, L, I, U, E, 0, 0]},
        {"name": "party", "curve": [61, 61, 71, 71, 71, 71, 71, 71, 61, 61], "buffer": [0, P, A, R, T1, T2, Y, 0]},
        {"name": "pop", "curve": [74, 65, 61, 60, 64, 73, 75, 75, 74, 74], "buffer": [0, 0, P, O, P, 0, 0, 0]},
        {"name": "reggae", "curve": [71, 71, 72, 81, 71, 62, 62, 71, 71, 71], "buffer": [0, R, E, G, G, A, E, 0]},
        {"name": "rock", "curve": [58, 63, 80, 84, 77, 66, 58, 55, 55, 55], "buffer": [0, 0, R, O, C, H, 0, 0]},
        {"name": "ska", "curve": [75, 79, 78, 72, 66, 63, 58, 57, 55, 57], "buffer": [0, 0, S, H, A, 0, 0, 0]},
        {"name": "soft_rock", "curve": [66, 66, 69, 72, 78, 80, 77, 72, 68, 58], "buffer": [S, O, F, T2, R, O, C, H]},
        {"name": "soft", "curve": [65, 70, 73, 75, 73, 66, 59, 57, 55, 53], "buffer": [0, S, O, F, T1, T2, 0, 0]},
        {"name": "techno", "curve": [60, 63, 71, 80, 79, 71, 60, 57, 57, 58], "buffer": [T1, T2, E, C, H, N, O, 0]},
        {"name": "nobass3", "curve": [0, 4, 14, 23, 47, 70, 87, 87, 87, 88], "buffer": [N, O, 0, B, A, S, S, Symbols.NUMBER[3]]},
        {"name": "nobass2", "curve": [0, 4, 14, 23, 47, 67, 76, 76, 76, 76], "buffer": [N, O, 0, B, A, S, S, Symbols.NUMBER[2]]},
        {"name": "nobass", "curve": [0, 4, 14, 23, 47, 65, 65, 65, 65, 65], "buffer": [N, O, 0, B, A, S, S, 0]}
    ]

    def __init__(self, core, default_tracks, preset):
        self._core = core
        self._volume = self.get_volume(),
        self._default_tracks = list(default_tracks) if isinstance(default_tracks, tuple) else [default_tracks]
        self._preset = preset

    def is_playing(self, state=None):
        if (state is None):
            state = self.get_state()
        return state == core.PlaybackState.PLAYING

    def is_paused(self, state=None):
        if (state is None):
            state = self.get_state()
        return state == core.PlaybackState.PAUSED

    def is_stopped(self, state=None):
        if (state is None):
            state = self.get_state()
        return state == core.PlaybackState.STOPPED

    def is_volume_changed(self, volume):
        if (self._volume != volume):
            self._volume = volume
            return True
        else:
            return False

    def is_mute(self):
        return self._core.mixer.get_mute().get()

    def play(self, tracks):
        if (tracks is not None):
            self._core.playback.stop()
            self._core.tracklist.clear()
            self._core.tracklist.add(uris=tracks)
            self._core.tracklist.set_consume(False)
            self._core.tracklist.set_single(False)
            self._core.tracklist.set_repeat(True)
            self._core.tracklist.set_random(True)
        if (not self.is_playing()):
            if (self._core.tracklist.get_length().get() < 1):
                self._core.tracklist.add(uris=self._default_tracks)
                self._core.tracklist.set_repeat(True)
                self._core.tracklist.set_random(True)
            self._core.playback.play()

    def pause(self):
        if (not self.is_paused()):
            self._core.playback.pause()

    def stop(self):
        if (self.is_playing()):
            self._core.playback.stop()

    def reboot(self):
        try:
            call("sudo shutdown -r now", shell=True)
        except Exception as inst:
            logging.error(inst)

    def halt(self):
        try:
            call("sudo shutdown -h now", shell=True)
        except Exception as inst:
            logging.error(inst)

    def run_sh(self):
        try:
            call(["sh", os.path.join(os.path.dirname(__file__), "run.sh")])
        except Exception as inst:
            logging.error(inst)

    def mute(self):
        self._core.mixer.set_mute(not self.is_mute())

    def set_preset(self, value):
        index = 0
        if (value == -1 or value == 1):
            index = (self._get_preset_index(self._preset) + value + len(self.PRESETS)) % len(self.PRESETS)
        else:
            index = self._get_preset_index(value)

        self._preset = self.PRESETS[index]["name"]
        try:
            curve = ""
            for i, c in enumerate(self.PRESETS[index]["curve"]):
                curve += "echo cset numid=%d %d;" % (i + 1, c)
            call("{ %s } | amixer -D equal -s" % curve, shell=True)
        except Exception as inst:
            logging.error(inst)
        return self.PRESETS[index]["buffer"]

    def get_presets(self):
        index = self._get_preset_index(self._preset)
        return self.PRESETS[index:len(self.PRESETS)] + self.PRESETS[0:index]

    def _get_preset_index(self, name):
        for i, p in enumerate(self.PRESETS):
            if (p["name"] == name):
                return i
        return 0

    def get_state(self):
        return self._core.playback.get_state().get()

    def get_volume(self):
        return self._core.mixer.get_volume().get()

    def set_volume(self, volume):
        if (volume < 0):
            volume = 0
        elif (volume > 100):
            volume = 100
        self._core.mixer.set_volume(volume)

    def increase_volume(self, volume=1):
        self.set_volume(self.get_volume() + volume)

    def decrease_volume(self, volume=1):
        self.set_volume(self.get_volume() - volume)

    def get_default_tracks(self):
        return self._default_tracks

    def get_draw_start_animation(self):
        return self.ANIMATION_START

    def get_draw_play_animation(self):
        return self.ANIMATION_PLAY

    def get_draw_stop_animation(self):
        return self.ANIMATION_STOP

    def get_draw_pause_animation(self):
        return self.ANIMATION_PAUSE

    def get_draw_seek_animation(self):
        return self.ANIMATION_SPINNER

    def get_draw_equalizer_animation(self):
        animation = []

        for j in range(120):
            frame = []
            for i in range(8):
                if (j > 0):
                    index = self.MUSIC_EQUALIZER.index(animation[j - 1][i]) + randint(-1, 1)
                    if (index < 0):
                        index = 0
                    elif (index > 3):
                        index = 3
                    frame.append(self.MUSIC_EQUALIZER[index])
                else:
                    frame.append(self.MUSIC_EQUALIZER[randint(0, 3)])
            animation.append(frame)

        return {
            "length": 3600,
            "repeat": 720,
            "sleep": 0.05,
            "buffer": animation
        }

    def get_draw_volume(self, volume=None):
        if (volume is None):
            volume = self.get_volume()

        # if (self.is_mute()):
        #     volume = 0

        return [
            Symbols.U,
            Symbols.O,
            Symbols.L,
            Symbols.NONE,
            Symbols.NONE,
            Symbols.NONE if volume < 100 else Symbols.NUMBER[int(volume / 100)],
            Symbols.NONE if volume < 10 else Symbols.NUMBER[int(volume / 10) % 10],
            Symbols.NUMBER[volume % 10]
        ]
